<?php
namespace ByDedi\AdminBundle\Controller;

use Sonata\AdminBundle\Controller\CRUDController as Controller;
use Sonata\AdminBundle\Datagrid\ProxyQueryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class CRUDController extends Controller
{
    public static $auditExclusion = [
        'setCreatedAt',
        'setUpdatedAt'
    ];

    public function __construct()
    {
        ini_set('memory_limit', '2G');
        set_time_limit(0);
    }


    /**
     * Edit action.
     *
     * @param int|string|null $id
     *
     * @return Response|RedirectResponse
     *
     * @throws NotFoundHttpException If the object does not exist
     * @throws AccessDeniedException If access is not granted
     */
    public function editAction($id = null)
    {
        $request = $this->getRequest();

        $id = $request->get($this->admin->getIdParameter());
        $existingObject = $this->admin->getObject($id);
        if (!$existingObject) {
            throw $this->createNotFoundException(sprintf('unable to find the object with id : %s', $id));
        }

        $this->admin->checkAccess('edit', $existingObject);

        $form = clone $this->admin->getForm();
        $form->setData($existingObject);
        $form->handleRequest($request);


        if ($form->isSubmitted()) {
            if (property_exists($this->admin, "hasValidationButtons")
                && $this->admin->hasValidationButtons === false
            ) {
                return $this->redirect($this->admin->generateObjectUrl('edit', $existingObject));
            }
        }

        return parent::editAction($id);
    }


    /**
     * List action.
     *
     * @return Response
     *
     * @throws AccessDeniedException If access is not granted
     */
    public function sublistAction()
    {
        $request = $this->getRequest();

        $this->admin->checkAccess('list');

        $preResponse = $this->preList($request);
        if ($preResponse !== null) {
            return $preResponse;
        }

        if ($listMode = $request->get('_list_mode')) {
            $this->admin->setListMode($listMode);
        }

        $datagrid = $this->admin->getDatagrid();
        $formView = $datagrid->getForm()->createView();

        // set the theme for the current Admin Form
        $this->get('twig')->getExtension('form')->renderer->setTheme($formView, $this->admin->getFilterTheme());

        return $this->render($this->admin->getTemplate('sublist'), array(
            'action' => 'list',
            'form' => $formView,
            'form_type_id' => $request->get('id'),
            'datagrid' => $datagrid,
            'origin_id' => $request->get('origin_id'),
            'pcode' => $request->get('pcode'),
            'csrf_token' => $this->getCsrfToken('sonata.batch'),
        ), null, $request);
    }

    /**
     * @return RedirectResponse
     */
    public function cloneAction($id = null)
    {
        $object = $this->admin->getSubject();

        if (!$object) {
            throw new NotFoundHttpException(sprintf('unable to find the object with id : %s', $id));
        }

        $this->admin->checkAccess('clone', $object);

        $clonedObject = clone $object;
        if (is_callable([$this->admin,'preClone'])) {
            call_user_func([$this->admin,'preClone'], $clonedObject);
        }
        $object = null;
        if (is_callable([$clonedObject,'getTranslations'])) {
            $translations = $clonedObject->getTranslations();
            foreach ($translations as $trans) {
                $newTranslations = clone($trans);
                $clonedObject->addTranslation($newTranslations);
            }
        }
        if (is_callable(array($clonedObject,'setName')) && is_callable(array($clonedObject,'getName'))) {
            $clonedObject->setName($clonedObject->getName() . ' (Clone)');
        }
        $this->admin->create($clonedObject);

        if (is_callable([$this->admin,'postClone'])) {
            call_user_func([$this->admin,'postClone'], $clonedObject);
        }

        $this->addFlash('sonata_flash_success', 'Cloned successfully');

        return new RedirectResponse($this->admin->generateUrl('edit', ['id' => $clonedObject->getId()]));
    }

    /**
     * @return null|Response
     */
    public function browseAction()
    {
        $request = $this->getRequest();

        $this->admin->checkAccess('list');

        $preResponse = $this->preList($request);
        if ($preResponse !== null) {
            return $preResponse;
        }

        if ($listMode = $request->get('_list_mode')) {
            $this->admin->setListMode($listMode);
        }

        $datagrid = $this->admin->getDatagrid();
        $formView = $datagrid->getForm()->createView();

        // set the theme for the current Admin Form
        $this->get('twig')->getExtension('form')->renderer->setTheme($formView, 'AdminBundle::admin/CRUD/browse.html.twig');

        return $this->render('AdminBundle::admin/CRUD/browse.html.twig', array(
            'action' => 'browse',
            'form' => $formView,
            'datagrid' => $datagrid,
            'csrf_token' => $this->getCsrfToken('sonata.batch'),
        ), null, $request);
    }


    public function historyAction($id = null)
    {
        $request = $this->getRequest();
        $id = $request->get($this->admin->getIdParameter());

        $object = $this->admin->getObject($id);

        if (!$object) {
            throw $this->createNotFoundException(sprintf('unable to find the object with id : %s', $id));
        }

        $this->admin->checkAccess('history', $object);

        $manager = $this->get('sonata.admin.audit.manager');

        if (!$manager->hasReader($this->admin->getClass())) {
            throw $this->createNotFoundException(
                sprintf(
                    'unable to find the audit reader for class : %s',
                    $this->admin->getClass()
                )
            );
        }

        $reader = $this->get('simplethings_entityaudit.reader');

        $revisions = $reader->findRevisions($this->admin->getClass(), $id);

        if (!empty($revisions)) {
            foreach ($revisions as $k => $r) {
                if (!empty($revisions[$k + 1])) {
                    $r->type = 'UPD';
                } else {
                    $r->type = 'INS';
                }
            }
        }

        return $this->render($this->admin->getTemplate('history'), array(
            'action' => 'history',
            'object' => $object,
            'revisions' => $revisions,
            'currentRevision' => $revisions ? current($revisions) : false,
        ), null, $request);
    }

    /**
     * @param ProxyQueryInterface $selectedModelQuery
     * @param Request             $request
     *
     * @return RedirectResponse
     */
    public function batchActionEnable(ProxyQueryInterface $selectedModelQuery, Request $request = null)
    {
        if (!$this->admin->isGranted('EDIT') || !$this->admin->isGranted('DELETE')) {
            throw new AccessDeniedException();
        }
        $modelManager = $this->admin->getModelManager();

        $selectedModelQuery->select('DISTINCT '.$selectedModelQuery->getRootAlias());

        try {
            $entityManager = $modelManager->getEntityManager($this->admin->getClass());
            $i = 0;
            foreach ($selectedModelQuery->getQuery()->iterate() as $pos => $object) {
                $object[0]->setStatus(true);
                $entityManager->persist($object[0]);
                if ((++$i % 20) == 0) {
                    $entityManager->flush();
                    $entityManager->clear();
                }
            }
            $entityManager->flush();
            $entityManager->clear();
        } catch (\Exception $e) {
            $this->addFlash('sonata_flash_error', 'flash_batch_merge_error');

            return new RedirectResponse(
                $this->admin->generateUrl('list', array('filter' => $this->admin->getFilterParameters()))
            );
        }

        $this->addFlash('sonata_flash_success', 'flash_batch_merge_success');

        return new RedirectResponse(
            $this->admin->generateUrl('list', array('filter' => $this->admin->getFilterParameters()))
        );
    }

    /**
     * @param ProxyQueryInterface $selectedModelQuery
     * @param Request             $request
     *
     * @return RedirectResponse
     */
    public function batchActionDisable(ProxyQueryInterface $selectedModelQuery, Request $request = null)
    {
        if (!$this->admin->isGranted('EDIT') || !$this->admin->isGranted('DELETE')) {
            throw new AccessDeniedException();
        }
        $modelManager = $this->admin->getModelManager();

        $selectedModelQuery->select('DISTINCT '.$selectedModelQuery->getRootAlias());

        try {
            $entityManager = $modelManager->getEntityManager($this->admin->getClass());
            $i = 0;
            foreach ($selectedModelQuery->getQuery()->iterate() as $pos => $object) {
                $object[0]->setStatus(false);
                $entityManager->persist($object[0]);
                if ((++$i % 20) == 0) {
                    $entityManager->flush();
                    $entityManager->clear();
                }
            }
            $entityManager->flush();
            $entityManager->clear();
        } catch (\Exception $e) {
            $this->addFlash('sonata_flash_error', 'flash_batch_merge_error');

            return new RedirectResponse(
                $this->admin->generateUrl('list', array('filter' => $this->admin->getFilterParameters()))
            );
        }

        $this->addFlash('sonata_flash_success', 'flash_batch_merge_success');

        return new RedirectResponse(
            $this->admin->generateUrl('list', array('filter' => $this->admin->getFilterParameters()))
        );
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function importAction(Request $request)
    {
        $action = $request->get('action');
        $method = 'import'.ucfirst($action);
        if (!is_callable([$this,$method])) {
            throw new NotFoundHttpException(sprintf('import method %s does not exist', $method));
        }

        return call_user_func([$this,$method], $request);
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function customExportAction(Request $request)
    {
        $action = $request->get('action');
        if (!is_callable([$this,$action])) {
            throw new NotFoundHttpException(sprintf('export method %s does not exist', $action));
        }

        return call_user_func([$this,$action], $request);
    }
}
